/*
*  Name:      test-1.uesql.cc
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Test for UESQL
*  Date:      $Date: 2000/08/02 22:30:09 $
*  Revision:  $Revision: 1.35 $
*
*  Copyright (C) 1996-1999  Rafael Jesus Alcantara Perez <rafa@dedalo-ing.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#include <cstdlib>
#include <iostream>
#include <mpcl/config_processor.hh>
#include <mpcl/string.hh>
#include <mpcl/test.h>


//
//  L O C A L   V A R I A B L E S
//

static uesql::TSqlstateDescriptor   _tSqlstateDescriptor;
  

//
//  L O C A L   F U N C T I O N S
//

/**
*  Construct error message from SQL status.
*  @return SQL status message.
*/
static mpcl::xstring _GetErrorMessage (void)
{

  using mpcl::xstring;

  return ( xstring (_tSqlstateDescriptor.getClass())    +
           ": "                                         +
           xstring (_tSqlstateDescriptor.getSubclass()) +
           " (" + uesql::SQLMESSAGE + ")"               );

}  // _GetErrorMessage()


int TestCursors (void)
{

  using mpcl::xstring;
  using uesql::TSqlString;

  TSqlString   yVar0;
  TSqlString   yVar1;
  TSqlString   yVar2;
  bool         gEmpty       = true;
  TSqlString   yAttribName  = "WORK_CODE";
  size_t       zRecordCount = 0;

  TEST_INIT ("test for cursors");

  extern "UESQL::default" 
  {
    whenever sqlerror  goto error;
    whenever not found goto loop_end;
    declare MYCURSOR cursor for
      select NAME, HOURS
        from WORKERS
        where ( NAME = 'nobody' );
    open MYCURSOR;
  }
  while (true)
  {
    extern "UESQL::default"
    {
      fetch from MYCURSOR into :yVar0, :yVar1;
    }
    gEmpty = false;
  }

loop_end:
  extern "UESQL::default" 
  {
    close MYCURSOR;
  }
  if ( gEmpty )
  {
    TEST_PASS;
  }
  else
  {
    TEST_FAIL;
  }

  extern "UESQL::default"
  {
    --
    --  WHENEVER
    --
    whenever sqlerror  goto error;
    whenever not found goto loop_end_2;

    --
    --  DECLARE CURSOR/OPEN/WHENEVER
    --
    declare MYCURSOR cursor for select :yAttribName, NAME, HOURS from WORKERS;
    open MYCURSOR;
  }
    
  if ( hptTestTargetFile )
  {
    fprintf (hptTestTargetFile, "%s\n", "WORK_CODE NAME                                     HOURS");
    fprintf (hptTestTargetFile, "%s\n", "========= ======================================== =====");
  }
  while (true)
  {
    extern "UESQL::default"
    {
      fetch from MYCURSOR into :yVar0, :yVar1, :yVar2;
    }
    if ( hptTestTargetFile )
    {
      fprintf (hptTestTargetFile, "%9s %40s %5s\n", yVar0.c_str(), yVar1.c_str(), yVar2.c_str());
    }
    ++zRecordCount;
  }

loop_end_2:
  extern "UESQL::default"
  {
    --
    --  CLOSE CURSOR
    --
    close MYCURSOR;
  }
  goto end;

error:
  cerr << "SQLSTATE="    << uesql::SQLSTATE    << endl;
  cerr << "SQLCODE="     << uesql::SQLCODE     << endl;
  cerr << "DESCRIPTION=" << _GetErrorMessage() << endl;
  TEST_FAIL_WITH_CODE (uesql::SQLCODE);  

end:
  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;
  
}  // TestCursors()


int TestDates (void)
{

  TEST_INIT ("test for dates");

  uesql::TSqlString   yVar0;
  uesql::TSqlDate     tSqlDate;
  std::size_t         zRecordCount = 0;

  extern "UESQL::default" 
  {
    whenever sqlerror  goto error;
    whenever not found goto loop_end;
    create table TABLE_1
    (
      CODE       character (3) not null ,
      BIRTH_DATE date
    );
    insert into TABLE_1 values ('000', '1999-12-31');
    insert into TABLE_1 values ('001', '1970-01-01');
    insert into TABLE_1 values ('002', '2000-01-01');
    insert into TABLE_1 values ('003', :uesql::TSqlDate (2000, 12, 31));
    insert into TABLE_1 values ('004', :uesql::TSqlDate (2001,  1,  1));
    declare MYCURSOR cursor for
      select CODE, BIRTH_DATE from TABLE_1;
    open MYCURSOR;
  }

  //
  //  Writes the example table.
  //
  if ( hptTestTargetFile )
  {
    fprintf (hptTestTargetFile, "%s\n", "CODE BIRTH_DATE");
    fprintf (hptTestTargetFile, "%s\n", "==== ==========");
  }
  while (true)
  {
    extern "UESQL::default"
    {
      fetch from MYCURSOR into :yVar0, :tSqlDate;
    }
    if ( hptTestTargetFile )
    {
      fprintf (hptTestTargetFile, "%3s  %10s\n", yVar0.c_str(), tSqlDate.get().c_str());
    }
    ++zRecordCount;
  }

loop_end:
  extern "UESQL::default" 
  {
    close MYCURSOR;
    drop table TABLE_1 cascade;
  }
  if ( zRecordCount == 5 )
  {
    TEST_PASS;
  }
  else
  {
    TEST_FAIL;
  }
  goto end;

error:
  cerr << "SQLSTATE="    << uesql::SQLSTATE    << endl;
  cerr << "SQLCODE="     << uesql::SQLCODE     << endl;
  cerr << "DESCRIPTION=" << _GetErrorMessage() << endl;
  TEST_FAIL_WITH_CODE (uesql::SQLCODE);  

end:  
  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;

}  // TestDates()


/**
*  Main.
*
*  Parameters:
*
*    {-o, --dbms-server-name}=DBMS-SERVER-NAME
*    {-u, --dbms-user-name}=DBMS-USER-NAME
*    {-p, --dbms-user-password}=PASSWORD
*/
int main (int argc, const char* argv[])
{

  using mpcl::Format;
  using mpcl::TConfigProcessor;
  using mpcl::xstring;
  using uesql::TSqlString;
  
  TEST_INIT ("test-1");
  TEST_START_WATCH;

  TConfigProcessor   tConfig;
  TSqlString         yVar0;
  TSqlString         yVar1;
  TSqlString         yVar2;
  TSqlString         yCursorName;
  TSqlString         yDbmsUserLogin;
  TSqlString         yDbmsSchemaLocation;
  TSqlString         yDbmsServerName;
  const char*        pcDbmsServerName = getenv ("DBMS_SERVER_NAME");
  const char*        pcDbmsUserName   = getenv ("DBMS_USER_NAME");
  TSqlString*        pyDbmsUserLogin  = &yDbmsUserLogin;
  TSqlString         yCodeValue       = "002";

  try
  {
    //
    //  Checks environment variables.
    //
    if ( !pcDbmsServerName )
    {
      pcDbmsServerName = "localhost";
    }
    if ( !pcDbmsUserName )
    {
      pcDbmsUserName = getenv ("USER");
      if ( !pcDbmsUserName )
      {
        pcDbmsUserName = "";
      }
    }
    
    //
    //  Define string constants names for options.
    //
    tConfig.addOption ("dbms-server-name",   "o", pcDbmsServerName);
    tConfig.addOption ("dbms-user-name",     "u", pcDbmsUserName);
    tConfig.addOption ("dbms-user-password", "p", pcDbmsUserName);
    tConfig.setCommandLineParameters (argc, argv);
    tConfig.processOptions();
    
    yDbmsServerName     = tConfig ["dbms-server-name"].value();
    yDbmsUserLogin      = tConfig ["dbms-user-name"].value();
    yDbmsSchemaLocation = uesql::ComposeSchemaLocation (yDbmsServerName, tConfig ["dbms-user-name"].value());
    if ( !tConfig ["dbms-user-password"].value().empty() )
    {
      yDbmsUserLogin = uesql::ComposeUserLogin
                       (
                         yDbmsUserLogin                         ,
                         tConfig ["dbms-user-password"].value() 
                       );
    }
  }
  catch (const mpcl::TBaseException& rktSOURCE_EXCEPTION)
  {
    cerr << rktSOURCE_EXCEPTION;
    TEST_FAIL_WITH_CODE (3);
  }

  TEST_WRITE_INFO ("DBMS-SCHEMA-LOCATION = %s", yDbmsSchemaLocation.c_str());
  TEST_WRITE_INFO ("DBMS-USER-LOGIN      = %s", yDbmsUserLogin.c_str());
  if ( !yDbmsServerName.empty() && !yDbmsUserLogin.empty() )
  {
    extern "UESQL::default" 
    {
      --
      --  WHENEVER
      --
      whenever sqlerror goto error;
      whenever not found goto not_found;
      
      --
      --  CONNECT TO
      --
      connect to :uesql::Quote (yDbmsSchemaLocation)
        user :uesql::Quote (yDbmsUserLogin);

      --
      --  CREATE TABLE
      --
      create table JOBS
      (
        CODE         character (3) not null ,
        WORKER_COUNT integer
      );

      create table WORKERS
      (
        WORK_CODE character (3)  not null ,
        NAME      character (40) not null ,
        HOURS     decimal (5)
      );

      --
      --  ALTER TABLE
      --
      alter table WORKERS add column STATE character (40);
      alter table WORKERS add column PHONE_NUMBER integer;

      --
      --  DELETE
      --
      delete from JOBS where CODE = :uesql::Quote (yCodeValue);
      delete from JOBS where CODE = '001';
      delete from WORKERS;

      --
      --  INSERT
      --
      insert into JOBS values ('001', 100);
      insert into JOBS values ('002', 250);
      insert into WORKERS values ('001', 'myname',   40, 'Andalucia',   5550000);
      insert into WORKERS values ('001', 'yourname', 35, 'Extremadura', 5550001);
      insert into WORKERS values ('001', 'hisname',  25, 'Catalunya',   5550002);
      insert into WORKERS values ('002', 'john',     35, 'New England', 5550003);
      insert into WORKERS values ('002', 'james',    35, 'Washington',  5550004);
      insert into WORKERS select WORK_CODE, NAME, HOURS, STATE, PHONE_NUMBER from WORKERS where NAME = 'myname';
      insert into WORKERS select WORK_CODE, NAME, HOURS, STATE, PHONE_NUMBER from WORKERS;

      --
      --  WHENEVER
      --
      whenever sqlerror goto cardinality_check;

      --
      --  SELECT for a single row.
      --
      --    Cardinality violation, so ( SQLSTATE == '21000' ).
      --
      select HOURS into :yVar1 from WORKERS, JOBS where WORK_CODE = CODE;
    }
    TEST_FAIL;

cardinality_check:
    TEST_STRINGS (uesql::SQLSTATE.c_str(), "21000");
    yCursorName = "MYCURSOR";
    extern "UESQL::default" 
    {
      --
      --  WHENEVER
      --
      whenever sqlerror goto error;

      --
      --  SELECT for a single row.
      --
      select count (*) into :yVar2 from WORKERS where name = 'myname';

      --
      --  UPDATE
      --
      update WORKERS
      set PHONE_NUMBER = WORKERS.PHONE_NUMBER
      where WORKERS.NAME = 'myname' and WORKERS.NAME = NAME;
    }
    TEST_FUNCTION (TestCursors());
    TEST_FUNCTION (TestDates());
    extern "UESQL::default" 
    {
      --
      --  DECLARE CURSOR/OPEN/FETCH/CLOSE
      --
      declare MYCURSOR cursor for select CODE from JOBS where CODE = '001';
      open MYCURSOR;
      fetch from :yCursorName into :yVar0;
      close MYCURSOR;
    }
    TEST_STRINGS (uesql::SQLSTATE.c_str(), "00000");
    TEST_STRINGS (yVar2.c_str(),           "4");
    TEST_STRINGS (yVar0.c_str(),           "001");
    extern "UESQL::default" 
    {
      --
      --  REVOKE
      --
      revoke all privileges on JOBS from public cascade;
      revoke all privileges on WORKERS from public restrict;

      --
      --  GRANT
      --
      grant select on JOBS    to public;
      grant select on WORKERS to public;
      grant delete on WORKERS to public;

      --
      --  CREATE VIEW
      --
      create view VIEW1 as select * from JOBS;

      --
      --  DROP VIEW
      --
      drop view VIEW1 cascade;

      --
      --  DROP TABLE
      --
      drop table JOBS cascade;

      --
      --  DISCONNECT
      --
      disconnect current;
    }

    extern "UESQL::default"
    {
      --
      --  CONNECT TO
      --
      connect to :uesql::Quote (yDbmsSchemaLocation)
        as 'myconnection'
        user :uesql::Quote (*pyDbmsUserLogin);

      --
      --  DROP TABLE
      --
      drop table WORKERS cascade;
    }
    goto end;

  error:
    cerr << "SQLSTATE="    << uesql::SQLSTATE    << endl;
    cerr << "SQLCODE="     << uesql::SQLCODE     << endl;
    cerr << "DESCRIPTION=" << _GetErrorMessage() << endl;
    TEST_FAIL_WITH_CODE (uesql::SQLCODE);
    goto end;

  not_found:
    cerr << "Search failed\n";
    TEST_FAIL_WITH_CODE (2);
    goto end;
  }

end:
  TEST_STOP_WATCH;
  TEST_DISPLAY_WATCH ("UESQL benchmark");
  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;

}  // main()
